open Format

exception Not_implemented

module List = struct
  include List

  let rec interval i j =
    if i <= j then i :: interval (succ i) j
    else []

  let format separator format_item fmt l =
    match l with
      | [] -> ()
      | h :: t ->
          format_item fmt h;
          let print item =
            fprintf fmt "%s%a" separator format_item item in
          iter print t
end

module Threadref = struct
  type 'a t = {
    mutable contents: 'a;
    mutex: Mutex.t;
  }

  let make contents = {
    contents = contents;
    mutex = Mutex.create ();
  }

  let get x =
    Mutex.lock x.mutex;
    let v = x.contents in
    Mutex.unlock x.mutex;
    v

  let set x v =
    Mutex.lock x.mutex;
    x.contents <- v;
    Mutex.unlock x.mutex

  let apply f x =
    Mutex.lock x.mutex;
    let v = x.contents in
    x.contents <- f v;
    Mutex.unlock x.mutex;
    v

  let incr x = apply succ x
end

module Array = struct
  include Array

  exception Not_equal

  let equal f a0 a1 =
    let l = length a0 in
    if length a1 = l then
      try
        for i = 0 to l - 1 do
          if not (f a0.(i) a1.(i)) then
            raise Not_equal
        done;
        true
      with Not_equal -> false
    else
      false

  let rec compare_from compare_item a1 a2 i =
    if i < Array.length a1 then
      if i < Array.length a2 then
        match compare_item a1.(i) a2.(i) with
          | 0 -> compare_from compare_item a1 a2 (succ i)
          | c -> c
      else
        1
    else if i < Array.length a2 then
      -1
    else
      0

  let compare compare_item a1 a2 =
    compare_from compare_item a1 a2 0

  let format separator format_item fmt a =
    if Array.length a >= 1 then
      begin
        format_item fmt a.(0);
        for i = 1 to Array.length a - 1 do
          fprintf fmt "%s%a" separator format_item a.(i)
        done
      end

  exception Exists

  let exists f a =
    try
      for i = 0 to Array.length a - 1 do
        if f a.(i) then
          raise Exists
      done;
      false
    with Exists -> true
end

module Set = struct
  module type OrderedType = Set.OrderedType

  module type S = sig
    include Set.S

    val of_list: elt list -> t

    val format: string -> (formatter -> elt -> unit) -> formatter -> t -> unit
  end

  module Make (Ord: OrderedType): S with type elt = Ord.t = struct
    include Set.Make (Ord)

    let add' set item = add item set

    let of_list l =
      List.fold_left add' empty l

    let format separator format_item fmt s =
      List.format separator format_item fmt (elements s)
  end
end

module Map = struct
  module type OrderedType = Map.OrderedType

  module type S = sig
    include Map.S
  end

  module Make (Ord: OrderedType): S with type key = Ord.t = struct
    include Map.Make (Ord)
  end
end

module Hashtbl = struct
  module Poly = Hashtbl

  module type HashedType = Hashtbl.HashedType

  let hash = Hashtbl.hash

  module Pair (H0: HashedType) (H1: HashedType): HashedType = struct
    type t = H0.t * H1.t

    let equal (x0, x1) (y0, y1) = H0.equal x0 y0 && H1.equal x1 y1

    let hash (x, y) = Hashtbl.hash (H0.hash x, H1.hash y)
  end

  module type S = sig
    include Hashtbl.S

    val tabulate: int -> (key -> 'a) -> key -> 'a
  end

  module Make (H: HashedType): S with type key = H.t = struct
    include Hashtbl.Make (H)

    let tabulate sz f =
      let tbl = create sz in
      let f' x =
        try find tbl x
        with Not_found ->
          let y = f x in
          add tbl x y;
          y in
      f'
  end
end

module Option = struct
  type 'a t = 'a option

  let map f item =
    match item with
      | None -> None
      | Some contents -> Some (f contents)

  let iter f item =
    match item with
      | None -> ()
      | Some contents -> f contents

  exception Get

  let get item =
    match item with
      | None -> raise Get
      | Some contents -> contents

  let find find item =
    try Some (find item)
    with Not_found -> None
end

module Int = struct
  module This = struct
    type t = int

    let compare: t -> t -> int = Pervasives.compare

    let equal: t -> t -> bool = ( = )

    let hash (x: t): int = x
  end

  include This

  module Set = Set.Make (This)

  module Map = Map.Make (This)

  module Hashtbl = Hashtbl.Make (This)

  let rec iter f i v =
    if i > 0 then iter f (pred i) (f v)
    else v
end

let pp_print_int_set fmt s =
  fprintf fmt "{%a}" (Int.Set.format ";" pp_print_int) s

module String = struct
  module This = struct
    include String

    let hash = Hashtbl.hash

    let equal = ( = )
  end

  include This

  let prefix_match u s =
    let ul = length u in
    length s >= ul && u = sub s 0 ul

  let remove_prefix l s = sub s l (length s - l)

  let suffix_match u s =
    let ul = length u and sl = length s in
    sl >= ul && u = sub s (sl - ul) ul

  let remove_suffix l s = sub s 0 (length s - l)

  let suffix_from s i = sub s i (length s - i)

  let rec split_from c s i =
    match Option.find (index_from s i) c with
      | None -> [suffix_from s i]
      | Some j ->
          sub s i (j - i) :: split_from c s (succ j)

  let split c s =
    split_from c s 0

  let to_array s =
    Array.init (length s) (get s)

  let of_char c = make 1 c

  module Set = Set.Make (This)

  module Map = Map.Make (This)

  module Hashtbl = Hashtbl.Make (This)
end

module Pair = struct
  let map f0 f1 (x0, x1) =
    (f0 x0, f1 x1)

  let compare compare1 compare2 (x0, x1) (y0, y1) =
    match compare1 x0 y0 with
      | 0 -> compare2 x1 y1
      | c -> c
end

module Utf8 = struct
  include Utf8

  module This = struct
    type t = string

    let compare = String.compare

    let hash = Hashtbl.hash

    let equal = ( = )
  end

  include This

  let of_latin1 s : t =
    let codes = Array.map int_of_char (String.to_array s) in
    from_int_array codes 0 (Array.length codes)

  module Hashtbl = Hashtbl.Make (This)

  module Set = Set.Make (This)

  module Map = Map.Make (This)
end

module Ref = struct
  let post_incr r =
    let value = !r in
    r := value + 1;
    value
end

module Char = struct
  module This = struct
    type t = char

    let hash = Hashtbl.hash

    let equal = ( = )

    let compare = Pervasives.compare
  end

  include This

  module Hashtbl = Hashtbl.Make (This)
end

let read_file filename f =
  let chan = open_in filename in
    try
      let result = f chan in
        close_in chan;
        result
    with exc ->
      close_in_noerr chan;
      raise exc

let write_file filename f =
  let chan = open_out filename in
    try
      let result = f chan in
        close_out chan;
        result
    with exc ->
      close_out_noerr chan;
      raise exc

type empty = { elim: 'a. 'a }

let rec y_or_n_loop () =
  let l = read_line () in
  if l = "y" then true
  else if l = "n" then false
  else
    begin
      fprintf err_formatter "'y' or 'n'? ";
      pp_print_flush err_formatter ();
      y_or_n_loop ()
    end


let y_or_n s =
  kfprintf
    (fun fmt ->
       fprintf fmt " (y/n)? ";
       pp_print_flush fmt ();
       y_or_n_loop ()
    ) err_formatter s
